package  
{
	/**
	 * GManager
	 * Purpose: To enable detection of gesture sequences.
	 * @author Shiu	27 October 2011
	 */
	public class GManager
	{
		private var gestSeq:Vector.<Vector.<int>>;	//Geture sequence definitions to be compared against
		private var gestName:Vector.<String>;			//Gesture sequence names
		private var strokes:Vector.<int>;					//Stokes allowed: main four only (1), diagonals only (2), else null
		private var gestCurrent:Vector.<int>;				//Current sequence made by user
		private var orderCurrent:int = 0;						//Current sequence to be compared
		private var _order:Boolean = false;					//Comparison done in order or not
		
		/**
		 * Constructor to initiate gestSeq, stokes, gestName
		 */
		public function GManager(){
			gestSeq = new Vector.<Vector.<int>>;
			gestName = new Vector.<String>;
			strokes = new Vector.<int>;
		}
		
		/**
		 * Method to register a set/ sequence of gesture
		 * @param	sequence	Numerical representation of gesture sequence
		 * @param	identifier		Name representation of gesture
		 * @param	onlyStrokes		Stokes allowed: Both (0), main four (1), diagonals (2)
		 */
		public function register(sequence:Vector.<int>, name:String, onlyStrokes:int = 0):void {
			gestSeq.push(sequence);
			gestName.push(name);
			strokes.push(onlyStrokes);
		}
		
		/**
		 * Method to remove a specific set of gesture
		 * @param	selectSet		Selected sequence to remove
		 */
		public function remove(selectSet:int):void {
			gestSeq.splice(selectSet, 1);
			gestName.splice(selectSet, 1);
			strokes.splice(selectSet, 1);
		}
		
		/**
		 * Method to remove all predefined gestures.
		 */
		public function removeAll():void {
			gestSeq = new Vector.<Vector.<int>>;
			gestName = new Vector.<String>;
			strokes = new Vector.<int>;
		}
		
		/**
		 * Property to get total sets of predefined gestures
		 */
		public function get length ():int {
			return gestSeq.length;
		}
		
		/**
		 * Property to get current setting on sequential gesture detection
		 */
		public function get useOrder():Boolean {
			return _order;
		}
		
		/**
		 * Property to determine whether sets of predefined gestures should be detected sequentially
		 */
		public function set useOrder(value:Boolean):void {
			_order = value;
		}
		
		/**
		 * Method to start recording current set of gesture made
		 */
		public function start():void {
			gestCurrent = new Vector.<int>;
		}
		
		/**
		 * Method to populate gesture into current set of gesture
		 * @param	gestureCode	Gesture detected from MGesture
		 */
		public function populate (gestureCode:int):void {
			//Accept only valid singular gesture (0~8) into current sequence
			if(gestureCode > -1)	gestCurrent.push(gestureCode);
		}
		
		/**
		 * Method used for debugging. Traces gestCurrent
		 */
		public function tracer():Vector.<int> {
			trace(gestCurrent);
			return gestCurrent;
		}
		
		/**
		 * Method to rid irrelevant strokes based upon condition
		 * @param	strokesCondition
		 * @return	Vector of 
		 */
		private function dropStrokes(inputArray:Vector.<int>, strokesCondition:int):Vector.<int> {
			var xDiagonals:Vector.<int> = inputArray.slice(0,inputArray.length);
			
			//Keep mains only
			if (strokesCondition == 1) {
				for (var i:int = 0; i < xDiagonals.length; i++) {
					if (xDiagonals[i] > 3) {
						xDiagonals.splice(i, 1);
						i--
					}
				}
			}
			
			else if (strokesCondition == 2) {
				for (var j:int = 0; j < xDiagonals.length; j++) {
					if (xDiagonals[j] <4) {
						xDiagonals.splice(j, 1);
						j--
					}
				}
			}
			
			return xDiagonals
		}
		
		/**
		 * Method to rid duplicates and possible mistaken gestures
		 * @param	minDuplicates		Repetition to be considered high repetition
		 * @return	Vector array w/o duplicates and possible mistaken gestures
		 */
		private function dropDuplicates (inputArray:Vector.<int>, minDuplicates:int = 1):Vector.<int> {
			
			//Append end of line indicator to integer array
			var xDuplicates:Vector.<int> = inputArray.slice(0,inputArray.length);
			xDuplicates.push( -1);
			
			//Initiate variables
			var count:int = 1;
			var keepIndex:int = 0;
			
			//Ripping duplicates and invalid moves
			for (var i:int = 0; i < xDuplicates.length - 1; i++) { 
				if (xDuplicates[i] == xDuplicates[i + 1])	count ++
				else{
					if (count > minDuplicates) { 
						xDuplicates.splice(keepIndex, count - 1)
						keepIndex++
						i = keepIndex
						count = 1
					}
					else		xDuplicates.splice(keepIndex, minDuplicates)
					i--
				}
			}
			
			xDuplicates.splice(xDuplicates.length - 1, 1);
			return xDuplicates;
		}
		
		/**
		 * Method to check if gestCurrent matches with gestSeq
		 * @param	index	Member of gestSeq
		 * @return	Match (1) or fail (-1)
		 */
		private function checkMatch(index:int):int {
			//Pessimistic search, assumes not found initially
			var matched:int = -1;
			
			//clean input first
			var cleaned:Vector.<int> = dropStrokes(gestCurrent, strokes[index]);
			cleaned = dropDuplicates(cleaned); trace(cleaned);
			
			//Check only those of same length
			if (cleaned.length == gestSeq[index].length) {
				var counter:int = cleaned.length;
				
				//Scroll through each integer of selected predefined gesture
				for (var member:int = 0; member < cleaned.length; member++) 
				{
					if (cleaned[member] == gestSeq[index][member]) counter--;
					else break;
				}
				
				//if all integers matched, current index of gestSeq matches gestCurrent
				if (counter == 0 ) matched = index;
			}
			
			return matched;
		}
		
		/**
		 * Method to evaluate gesture according to preferred approach (in sequential order or not)
		 * @return	No predefined gestures (-2), No match of gesture (-1), array with index of match and its description
		 */
		public function end():Array {
			//Pessimistic search, -2 indicates that there are no predefined gesture
			var result:Array = [ -2, "No predefined gestures"];
			
			//If there are predefined gestures
			if (gestSeq.length > 0) {
				
				//Find the match in order
				if (_order) {
					if (checkMatch(orderCurrent) > -1) {
						result = [orderCurrent, gestName[orderCurrent]];
						orderCurrent++;
					}
				}
				
				//Find a match in no specific order
				else if (!_order) {
					//Scroll through all gestures
					for (var i:int = 0; i < gestSeq.length; i++) 
					{
						if (checkMatch(i) > -1) {
							result = [i, gestName[i]];
							break;
						}
					}
				}
				
				//Handle when cant find
				else {
					result = [ -1, "Gesture is invalid."]
				}
			}
			return result
		}
	}
}